<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    INVERTUS, UAB www.invertus.eu <support@invertus.eu>
 * @copyright Copyright (c) permanent, INVERTUS, UAB
 * @license   Addons PrestaShop license limitation
 * @see       /LICENSE
 *
 * International Registered Trademark & Property of INVERTUS, UAB
 */

namespace Invertus\SwedBank\Service;

use Invertus\SwedBank\Adapter\Configuration;
use Invertus\SwedBank\Adapter\Link;
use Invertus\SwedBank\Config\BankLink;
use Invertus\SwedBank\Config\Config;
use PrestaShop\PrestaShop\Core\Payment\PaymentOption;
use RuntimeException;
use SwedBank;
use Translate;

class PaymentOptionProvider
{
    /**
     * @var Configuration
     */
    private $configuration;

    /**
     * @var Link
     */
    private $link;

    /**
     * @var array   Additional params added to each payment option URL
     */
    private $additionalParams = [];

    /**
     * PaymentOptionProvider constructor.
     *
     * @param Configuration $configuration
     * @param Link $link
     */
    public function __construct(Configuration $configuration, Link $link)
    {
        $this->configuration = $configuration;
        $this->link = $link;
    }

    /**
     * Get available payment options
     *
     * @param string $environment       Must be 'test' or 'live'
     * @param array $additionalParams   Additional params added to each payment option URL
     *
     * @return PaymentOption[]
     */
    public function getPaymentOptions($environment, array $additionalParams = [])
    {
        $this->additionalParams = $additionalParams;

        if (SwedBank::ENV_TEST == $environment) {
            $paymentOptions = $this->getFormattedPaymentOptions('SB_PAYMENT_METHODS_TEST');
        } elseif (SwedBank::ENV_LIVE == $environment) {
            $paymentOptions = $this->getFormattedPaymentOptions('SB_PAYMENT_METHODS_LIVE');
        } else {
            throw new RuntimeException(sprintf('Invalid "%s" environment given.', $environment));
        }

        return $paymentOptions;
    }

    /**
     * @param string $configName
     *
     * @return PaymentOption[]
     */
    private function getFormattedPaymentOptions($configName)
    {
        $paymentOptions = $this->configuration->get($configName);
        $paymentOptions = json_decode($paymentOptions, true);

        $formattedPaymentOptions = [];

        foreach ($paymentOptions as $idPaymentOption) {
            switch ($idPaymentOption) {
                case Config::PAYMENT_METHOD_CARD:
                    $name = $this->l('Pay by Card');
                    $description =
                        $this->l('After choosing this payment method you will be redirected to payment provider page');
                    $url = $this->link->getModuleLink('hps', $this->additionalParams);
                    break;
                case Config::PAYMENT_METHOD_SWEDBANK:
                    $name = $this->l('Pay by Swedbank');
                    $description =
                        $this->l('After choosing this payment method you will be redirected to payment provider page');
                    $params =
                        array_merge(['payment_method' => BankLink::PAYMENT_METHOD_SWEDBANK], $this->additionalParams);
                    $url = $this->link->getModuleLink('banklink', $params);
                    break;
                case Config::PAYMENT_METHOD_DNB:
                    $name = $this->l('Pay by DNB');
                    $description =
                        $this->l('After choosing this payment method you will be redirected to payment provider page');
                    $params = array_merge(['payment_method' => BankLink::PAYMENT_METHOD_DNB], $this->additionalParams);
                    $url = $this->link->getModuleLink('banklink', $params);
                    break;
                case Config::PAYMENT_METHOD_SEB:
                    $name = $this->l('Pay by SEB');
                    $description =
                        $this->l('After choosing this payment method you will be redirected to payment provider page');
                    $params = array_merge(['payment_method' => BankLink::PAYMENT_METHOD_SEB], $this->additionalParams);
                    $url = $this->link->getModuleLink('banklink', $params);
                    break;
                case Config::PAYMENT_METHOD_NORDEA:
                    $name = $this->l('Pay by Nordea');
                    $description =
                        $this->l('After choosing this payment method you will be redirected to payment provider page');
                    $params =
                        array_merge(['payment_method' => BankLink::PAYMENT_METHOD_NORDEA], $this->additionalParams);
                    $url = $this->link->getModuleLink('banklink', $params);
                    break;
                case Config::PAYMENT_METHOD_DANSKE:
                    $name = $this->l('Pay by Danske');
                    $description =
                        $this->l('After choosing this payment method you will be redirected to payment provider page');
                    $params =
                        array_merge(['payment_method' => BankLink::PAYMENT_METHOD_DANSKE], $this->additionalParams);
                    $url = $this->link->getModuleLink('banklink', $params);
                    break;
                case Config::PAYMENT_METHOD_CITADELE:
                    $name = $this->l('Pay by Citadele');
                    $description =
                        $this->l('After choosing this payment method you will be redirected to payment provider page');
                    $params =
                        array_merge(['payment_method' => BankLink::PAYMENT_METHOD_CITADELE], $this->additionalParams);
                    $url = $this->link->getModuleLink('banklink', $params);
                    break;
                case Config::PAYMENT_METHOD_PAYPAL:
                    $name = $this->l('Pay by Paypal');
                    $description =
                        $this->l('After choosing this payment method you will be redirected to payment provider page');
                    $url = $this->link->getModuleLink('paypal', $this->additionalParams);
                    break;
                default:
                    throw new RuntimeException(sprintf('Invalid "%s" payment option supplied', $idPaymentOption));
            }

            $paymentOption = new PaymentOption();
            $paymentOption->setCallToActionText($name);
            $paymentOption->setAction($url);
            $paymentOption->setAdditionalInformation($description);

            $formattedPaymentOptions[] = $paymentOption;
        }

        return $formattedPaymentOptions;
    }

    /**
     * Translates text
     *
     * @param string $text
     *
     * @return string
     */
    protected function l($text)
    {
        return Translate::getModuleTranslation('swedbank', $text, 'PaymentOptionProvider');
    }
}
